// ==============================================================
//                 ORBITER MODULE: ARROWHANGAR
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// ARROWHANGAR.cpp
// Control module for ARROWHANGAR vessel class
//

// ==============================================================
#define ORBITER_MODULE
#include "orbitersdk.h"
#include "ARROWHANGAR2.h"
#include "ARROWMESH.h"

//
#include "OrbiterSoundSDK35.h"
VISHANDLE MainExternalMeshVisual = 0;






char *ARROWHANGAR2::SendHudMessage()
{
dHudMessageDelay=15;
return cUmmuHudDisplay;
}



void ARROWHANGAR2::clbkPostCreation (void)

{
{
	DWORD mode=GetADCtrlMode ();
	mode|=32;
	SetADCtrlMode (mode);
}
ARROWHANGARSoundID=ConnectToOrbiterSoundDLL3(GetHandle());
ReplaceStockSound3(ARROWHANGARSoundID,"Sound\\Vessel\\MOONBASEALPHA\\ambient.wav",	REPLACE_COCKPIT_AMBIENCE_1);
SoundOptionOnOff3(ARROWHANGARSoundID,PLAYMAINTHRUST,FALSE);

SoundOptionOnOff3(ARROWHANGARSoundID,PLAYHOVERTHRUST,FALSE);
SoundOptionOnOff3(ARROWHANGARSoundID,PLAYATTITUDETHRUST,FALSE);

SoundOptionOnOff3(ARROWHANGARSoundID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);

SoundOptionOnOff3(ARROWHANGARSoundID,PLAYCABINAIRCONDITIONING,TRUE);

SoundOptionOnOff3(ARROWHANGARSoundID,PLAYCABINRANDOMAMBIANCE,TRUE);

SoundOptionOnOff3(ARROWHANGARSoundID,PLAYRADIOATC,FALSE);

SoundOptionOnOff3(ARROWHANGARSoundID,DISPLAYTIMER,FALSE);
//SetTouchdownPoints (_V(0,-3.124+GEAR_proc*1.5,8.7), _V(-2,-3.124+GEAR_proc*1.5,-9.6), _V(2,-3.124+GEAR_proc*1.5,-9.6));

}


// Constructor
ARROWHANGAR2::ARROWHANGAR2 (OBJHANDLE hObj, int fmodel)
: VESSEL2 (hObj, fmodel)



{

DOOR0_status      = DOOR_UP;
DOOR0_proc        = 0.0;

DOOR1_status      = DOOR_UP;
DOOR1_proc        = 0.0;

DOOR2_status      = DOOR_UP;
DOOR2_proc        = 0.0;

DOOR3_status      = DOOR_UP;
DOOR3_proc        = 0.0;

DOOR4_status      = DOOR_UP;
DOOR4_proc        = 0.0;

DOOR5_status      = DOOR_UP;
DOOR5_proc        = 0.0;

DOOR6_status      = DOOR_UP;
DOOR6_proc        = 0.0;

DOOR7_status      = DOOR_UP;
DOOR7_proc        = 0.0;

DOOR8_status      = DOOR_UP;
DOOR8_proc        = 0.0;


LIFT_SPEED =.25;



DefineAnimations ();
	
}



//
void ARROWHANGAR2::clbkSetClassCaps (FILEHANDLE cfg)
//void ARROWHANGAR::SetTransporter()
{
	// physical specs
	SetSize (175);
	SetEmptyMass (10200.0);
	SetCW (0.3, 0.3, 0.6, 0.9);
	SetWingAspect (0.1);
	SetWingEffectiveness (0.1);
	SetCrossSections (_V(21.63,22.56,9.82));
	
	SetPMI (_V(22.05,22.56,1.49));
	SetTrimScale (0.05);
	SetCameraOffset (_V(0,10,-3));
	
	SetTouchdownPoints  (_V(0,.001,1), _V(-1.6,.001,-1), _V(1.6,.001,-1));; 
EnableTransponder (true);

// visual specs
TUBE = oapiLoadMeshGlobal ("ARROWHANGAR6");
BUG0 = AddMesh (TUBE);
SetMeshVisibilityMode (BUG0, MESHVIS_ALWAYS | MESHVIS_EXTPASS);

// Call clbkSetClassCaps_UMMU and select a member
	SelectedUmmuMember=0;
	clbkSetClassCaps_UMMu();

	
   

}

// =======================================================
// UMMU Class Caps
// =======================================================

void ARROWHANGAR2::clbkSetClassCaps_UMMu(void)
{
	//--------------------------------------------------------------
	// InitUmmu
	// initialisation of Ummu must be done first ! return of 1 mean OK 
	// return of -999 mean Ummu addon is not installed in user's Orbiter, return of -1 
	// mean misc error. If error simply all function will fail silently 
	// and there will be no MMu. It's a good idea to warn the user. 
	// (see function "WarnUserUMMUNotInstalled" below)
	Crew.InitUmmu(GetHandle());	
	//--------------------------------------------------------------

	//--------------------------------------------------------------
	// GetUserUMmuVersion
	// this check wich version of UMmu addon the user have in his orbiter directory.
	// this may be usefull for future version with more function so you can check
	// if user installation support the new functions. Return -1 on error (not installed, version cannot be read)
	// or version number in float value (ie: 2.0="2.0" version etc etc)
	float UMmuVersion=Crew.GetUserUMmuVersion();

	//--------------------------------------------------------------
	// DefineAirLockShape
	// We set the airlock shape and state, this virtual box will be the one
	// into were a Ummu asking to reenter will be taken in account
	// first parameter is airlock door state (OPEN=TRUE) and other are X,X1,Y,Y1 and Z,Z1 
	// min and max coordinate for this virtual box (vessel local coordinate)
	// Pay attention to reenter when landed (if suitable) and make the box large enough so 
	// it's not a pain for the user to find airlock's entry. (See PDF doc)
	Crew.DefineAirLockShape(TRUE,-99,-97,-10,10,-183,-179);			// Airlock open, 2 meter large 4 meter high 6 meter long (vessel local coordinate)

	//--------------------------------------------------------------
	// SetMembersPositionOrientationOnEVA
	// We define here were the Ummu will appear when they make an EVA
	// first parameter is position second is orientation, take care not to make
	// them appear to high on ground (will fall and die) or to low (Orbiter "light speed" bug)
	Crew.SetMembersPosRotOnEVA(_V( -97.7,.858,-180.7),_V(0,-1,0));	// 3 meters in front of ship (z) facing direction of ship (vessel local coordinate)
//-97.71086 1.494081 -180.7069
	//---------------------------------------------------------------
	// finally set the maximum seat available in this ship, default is
	// 8 but it can go from 1 to 100
	// BAD IDEA: using this to "close" your ship, use "SetAirlockDoorState" instead.
	Crew.SetMaxSeatAvailableInShip(4);

	//----------------------------------------------------------------
	// OPTIONAL OPTIONAL OPTIONAL OPTIONAL OPTIONAL OPTIONAL OPTIONAL
	// DeclareActionArea
	// Ever wanted to be able to open a door, repair a system or trigger something in your vessel from an UMMU ?
	// Here come a new feature of UMMU 2.0: Action area. See UMmuSDK.h and PDF doc for complete explanation and see
	// "DetectActionAreaActivated" below.
	Crew.DeclareActionArea(0,_V(-97.74154 ,0,-178.406),2,TRUE,"action_door.wav","Open/Close Inner Airlock Doors");
	Crew.DeclareActionArea(1,_V(-97.48835,0,-182.9494),2,TRUE,"action_door.wav","Open/Close Outer Airlock Doors");
// 149.0628 46.9635  120.1651
		Crew.DeclareActionArea(2,_V( 151.5493,0,112.2137),8,TRUE,"action_door.wav","Open/Close Storage1");
		Crew.DeclareActionArea(3,_V(  151.9841 ,0, 24.42339),8,TRUE,"action_door.wav","Open/Close Storage1");

				Crew.DeclareActionArea(4,_V(  151.3511,0,-28.44929),8,TRUE,"action_door.wav","Open/Close Storage2");
		Crew.DeclareActionArea(5,_V(   151.3511,0,-28.44929),8,TRUE,"action_door.wav","Open/Close Storage2");

				Crew.DeclareActionArea(6,_V( -151.4557,0,-29.06965),8,TRUE,"action_door.wav","Open/Close Storage3");
		Crew.DeclareActionArea(7,_V(   -151.4984,0,-151.2548),8,TRUE,"action_door.wav","Open/Close Storage3");

				Crew.DeclareActionArea(8,_V(  -151.991,0,112.2137),8,TRUE,"action_door.wav","Open/Close Storage4");
		Crew.DeclareActionArea(9,_V(  -152.0337,0,25.05727),8,TRUE,"action_door.wav","Open/Close Storage4");

						Crew.DeclareActionArea(10,_V( -14.9829,0,-181.1753),8,TRUE,"action_door.wav","Open/Close Rear door");
		Crew.DeclareActionArea(11,_V(   13.85118,0,-181.9311),8,TRUE,"action_door.wav","Open/Close Rear Door");

								Crew.DeclareActionArea(12,_V(  110.9091,0,144.4968),8,TRUE,"action_door.wav","Open/Close Front door");
		Crew.DeclareActionArea(13,_V(  -110.936,0,144.7169),8,TRUE,"action_door.wav","Open/Close Front Door");

	//Crew.DeclareActionArea(1,_V(54,0,0),5,TRUE, "action_door.wav","Open/Close Travel Tube Doors");
	//Crew.DeclareActionArea(2,_V(61,0,-11.75),2,TRUE, "action_door.wav","Inner AIrlock Active");
	//Crew.DeclareActionArea(3,_V(65.5,0,-11.75),2,TRUE, "action_door.wav","Outer AIrlock Active");

	//Crew.DeclareActionArea(3,_V(0,0,-5),2.5,TRUE,"action_activated.wav","You are behind the ship");
	//iActionAreaDemoStep=0;	// this is just to show a feature of action area, see below "DetectActionAreaActivated"

	//--------------------------------------------------------------
	// AddCrewMember
	// We'll add four default members for when the ship is spawned by Orbiter's scenario editor. 
	// parameters are name, age, cardiac pulse,weight (in kilogramm) and Function (Misc ID)
	// max four characters wich define the spacesuit used. (see UmmuMiscID in UMuSDK.h header)
	// Return 1 on success -1 on error (probably vessel full, name>25 char 
	// age, pulse or weight out of realistic range or MiscID>4 char)
	Crew.AddCrewMember("Peter Falcon",41,65,74,"Capt");		//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Fanny Gorgeous",27,67,55,"Eng");	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("George HealGood",15,70,45,"Doc");	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Albert Jr Falcon",15,70,45);		//(for name and id a-z A-Z 0-9 characters only)
	SelectedUmmuMember =0;  // our current selected member


	// The HUD display method variables, see PDF doc
	cUmmuHudDisplay[0] =0;	// Initialisation of UMmu hud char variable
	dHudMessageDelay =0;	// Initialisation of UMmu delay variable
	strcpy(SendHudMessage(),"Welcome aboard ! E=EVA 1,2=select UMmu D=Open/Close airlock S=info M=add crew");

	// The Add mmu without scenery editor variable see PDF doc
	cAddUMmuToVessel[0]=0;
}








void ARROWHANGAR2::clbkPostStep(double simt, double simdt, double mjd)
{
//---------------------------------------------------------------------------
	// ProcessUniversalMMu
	// Here the routine that detect if someone entered the ship (eva or transfer)
	// No need to manage anything here all is automatic, crew is added to your ship, ship's weight is updated,
	// and UMmu vessel is automatically deleted from Orbiter. You may just look the return
	// code to see if someone entered and display wathewer message on panel or other.
	// notice it's FPS friendly, function process only 4 time per second not each frame 
	// and return immediately if airlock closed or no Ummu is detected in vincinity. 
	int ReturnCode=Crew.ProcessUniversalMMu();
	switch(ReturnCode)
	{
	case UMMU_TRANSFERED_TO_OUR_SHIP: 
		sprintf(SendHudMessage(),"%s \"%s\" aged %i was transfered to our ship",
		Crew.GetCrewMiscIdByName(Crew.GetLastEnteredCrewName()),Crew.GetLastEnteredCrewName()
		,Crew.GetCrewAgeByName(Crew.GetLastEnteredCrewName()));
		break;
	case UMMU_RETURNED_TO_OUR_SHIP:
		sprintf(SendHudMessage(),"%s \"%s\" aged %i entered into our ship",
		Crew.GetCrewMiscIdByName(Crew.GetLastEnteredCrewName()),
		Crew.GetLastEnteredCrewName(),Crew.GetCrewAgeByName(Crew.GetLastEnteredCrewName()));
		break;
	}
	if(GroundContact()==TRUE)
	{
		// we check vertical speed
		//int I;
		VECTOR3 vHorizonAirspeedVector={0};
		GetHorizonAirspeedVector (vHorizonAirspeedVector);
		double VertSpeed		=vHorizonAirspeedVector.y;
		if(VertSpeed<-3)
		{
			// we touched ground with more than -3 m/s, sorry dude, time to kill you all :(
			//for(I=0;I<Crew.GetCrewTotalNumber();I++)
			//{
		//		Crew.SetCrewMemberPulseBySlotNumber(I,0);	// set cardiac pulse to zero
		//	}
		//	strcpy(SendHudMessage(),"Oooh no ! Crash - All crew aboard killed");
		}

		// TIPS: the vertical speed is often reset to zero when there is ground contact
		// this may bug somewhat the death of your crew.
		// to have an accurate VertSpeed at touchdown I recommand to record it at very END 
		// of timestep and use this "old" value. The next frame you'll have the vertspeed value 
		// of *last frame* just before the crash (GroundContact). This ensure an accurate 
		// verticalspeed value. (keep this value in your vessel class and don't forget to 
		// initialize it at zero in setclasscap)
	}

	int ActionAreaReturnCode=Crew.DetectActionAreaActivated();
	
	if(ActionAreaReturnCode>-1)
	{
		// this is just an example, we have four area declared
		// action area ID 0 triggered 

	
		if(ActionAreaReturnCode==0)
		{
			if (DOOR4_proc==0){
		RevertDOOR3();
			}}
if(ActionAreaReturnCode==1)
		{
			if (DOOR3_proc==0){
		RevertDOOR4();
			}}
if((ActionAreaReturnCode==2) ||(ActionAreaReturnCode==3))
		{
			
		RevertDOOR5();
			}
if((ActionAreaReturnCode==4) ||(ActionAreaReturnCode==5))
		{
			
		RevertDOOR6();
			}
if((ActionAreaReturnCode==6) ||(ActionAreaReturnCode==7))
		{
			
		RevertDOOR7();
			}
if((ActionAreaReturnCode==8) ||(ActionAreaReturnCode==9))
		{
			
		RevertDOOR8();
			}
if((ActionAreaReturnCode==10) ||(ActionAreaReturnCode==11))
		{
			
		RevertDOOR2();
			}

if((ActionAreaReturnCode==12) ||(ActionAreaReturnCode==13))
		{
			
		RevertDOOR();
			}


	}




//sprintf(oapiDebugString(),"anim %2.2f", DOOR0_proc);

	//---------------------------------------------------------------------------
	// WarnUserUMMUNotInstalled - IMPORTANT helper
	// Put here this function and users will be automatically warned if they don't have
	// UMMU installed or if it's outdated. Warning text duration is 20 seconds and text is:
	// [AddonName] require "Universal MMU" ver 2.0 or higher. Download at www.orbiter.dansteph.com (message countdown in seconds)
	// an additonal "your version of UMMU is outdated" is displayed the last 5 seconds if they have version lower than 2.0
//	Crew.WarnUserUMMUNotInstalled("ARROWHANGAR");



	// THIS IS FOR ADDING CREW SEE PDF doc "Allow user to add crew to your ship 
	// without scenery editor"
	AddUMmuToVessel();

if (DOOR0_status >= DOOR_RAISING) {
		double da = simdt * LIFT_SPEED;
		if (DOOR0_status == DOOR_RAISING) {
			if (DOOR0_proc > 0.0) DOOR0_proc = max (0.0, DOOR0_proc-da);
			else                DOOR0_status = DOOR_UP;
		} else {
			if (DOOR0_proc < 1.0) DOOR0_proc = min (1.0, DOOR0_proc+da);
			else                DOOR0_status = DOOR_DOWN;
		}


SetAnimation (anim_DOOR1, DOOR0_proc);		
SetAnimation (anim_DOOR2, DOOR0_proc);

SetAnimation (anim_DOOR3, DOOR0_proc);
SetAnimation (anim_DOOR4, DOOR0_proc);
SetAnimation (anim_DOOR5, DOOR0_proc);
SetAnimation (anim_DOOR6, DOOR0_proc);
SetAnimation (anim_DOOR7, DOOR0_proc);
SetAnimation (anim_DOOR8, DOOR0_proc);
SetAnimation (anim_DOOR9, DOOR0_proc);
SetAnimation (anim_DOOR10, DOOR0_proc);

}		

if (DOOR1_status >= DOOR_RAISING) {
		double da = simdt * LIFT_SPEED;
		if (DOOR1_status == DOOR_RAISING) {
			if (DOOR1_proc > 0.0) DOOR1_proc = max (0.0, DOOR1_proc-da);
			else                DOOR1_status = DOOR_UP;
		} else {
			if (DOOR1_proc < 1.0) DOOR1_proc = min (1.0, DOOR1_proc+da);
			else                DOOR1_status = DOOR_DOWN;
		}


SetAnimation (anim_DOOR11, DOOR1_proc);		
SetAnimation (anim_DOOR12, DOOR1_proc);

SetAnimation (anim_DOOR13, DOOR1_proc);
SetAnimation (anim_DOOR14, DOOR1_proc);
SetAnimation (anim_DOOR15, DOOR1_proc);
SetAnimation (anim_DOOR16, DOOR1_proc);
SetAnimation (anim_DOOR17, DOOR1_proc);
SetAnimation (anim_DOOR18, DOOR1_proc);
SetAnimation (anim_DOOR19, DOOR1_proc);
SetAnimation (anim_DOOR20, DOOR1_proc);

}		

if (DOOR2_status >= DOOR_RAISING) {
		double da = simdt * LIFT_SPEED;
		if (DOOR2_status == DOOR_RAISING) {
			if (DOOR2_proc > 0.0) DOOR2_proc = max (0.0, DOOR2_proc-da);
			else                DOOR2_status = DOOR_UP;
		} else {
			if (DOOR2_proc < 1.0) DOOR2_proc = min (1.0, DOOR2_proc+da);
			else                DOOR2_status = DOOR_DOWN;
		}


SetAnimation (anim_DOOR21, DOOR2_proc);		
SetAnimation (anim_DOOR22, DOOR2_proc);

}

if (DOOR3_status >= DOOR_RAISING) {
		double da = simdt * LIFT_SPEED;
		if (DOOR3_status == DOOR_RAISING) {
			if (DOOR3_proc > 0.0) DOOR3_proc = max (0.0, DOOR3_proc-da);
			else                DOOR3_status = DOOR_UP;
		} else {
			if (DOOR3_proc < 1.0) DOOR3_proc = min (1.0, DOOR3_proc+da);
			else                DOOR3_status = DOOR_DOWN;
		}


SetAnimation (anim_airlock1, DOOR3_proc);		

}

if (DOOR4_status >= DOOR_RAISING) {
		double da = simdt * LIFT_SPEED;
		if (DOOR4_status == DOOR_RAISING) {
			if (DOOR4_proc > 0.0) DOOR4_proc = max (0.0, DOOR4_proc-da);
			else                DOOR4_status = DOOR_UP;
		} else {
			if (DOOR4_proc < 1.0) DOOR4_proc = min (1.0, DOOR4_proc+da);
			else                DOOR4_status = DOOR_DOWN;
		}


SetAnimation (anim_airlock2, DOOR4_proc);		

}

//cargo1
if (DOOR5_status >= DOOR_RAISING) {
		double da = simdt * LIFT_SPEED;
		if (DOOR5_status == DOOR_RAISING) {
			if (DOOR5_proc > 0.0) DOOR5_proc = max (0.0, DOOR5_proc-da);
			else                DOOR5_status = DOOR_UP;
		} else {
			if (DOOR5_proc < 1.0) DOOR5_proc = min (1.0, DOOR5_proc+da);
			else                DOOR5_status = DOOR_DOWN;
		}


SetAnimation (anim_DOOR51, DOOR5_proc);		
SetAnimation (anim_DOOR52, DOOR5_proc);

SetAnimation (anim_DOOR53, DOOR5_proc);
}

if (DOOR6_status >= DOOR_RAISING) {
		double da = simdt * LIFT_SPEED;
		if (DOOR6_status == DOOR_RAISING) {
			if (DOOR6_proc > 0.0) DOOR6_proc = max (0.0, DOOR6_proc-da);
			else                DOOR6_status = DOOR_UP;
		} else {
			if (DOOR6_proc < 1.0) DOOR6_proc = min (1.0, DOOR6_proc+da);
			else                DOOR6_status = DOOR_DOWN;
		}


SetAnimation (anim_DOOR61, DOOR6_proc);		
SetAnimation (anim_DOOR62, DOOR6_proc);

SetAnimation (anim_DOOR63, DOOR6_proc);
SetAnimation (anim_DOOR64, DOOR6_proc);

}

if (DOOR7_status >= DOOR_RAISING) {
		double da = simdt * LIFT_SPEED;
		if (DOOR7_status == DOOR_RAISING) {
			if (DOOR7_proc > 0.0) DOOR7_proc = max (0.0, DOOR7_proc-da);
			else                DOOR7_status = DOOR_UP;
		} else {
			if (DOOR7_proc < 1.0) DOOR7_proc = min (1.0, DOOR7_proc+da);
			else                DOOR7_status = DOOR_DOWN;
		}


SetAnimation (anim_DOOR71, DOOR7_proc);		
SetAnimation (anim_DOOR72, DOOR7_proc);

SetAnimation (anim_DOOR73, DOOR7_proc);
SetAnimation (anim_DOOR74, DOOR7_proc);

}





//cargo4

if (DOOR8_status >= DOOR_RAISING) {
		double da = simdt * LIFT_SPEED;
		if (DOOR8_status == DOOR_RAISING) {
			if (DOOR8_proc > 0.0) DOOR8_proc = max (0.0, DOOR8_proc-da);
			else                DOOR8_status = DOOR_UP;
		} else {
			if (DOOR8_proc < 1.0) DOOR8_proc = min (1.0, DOOR8_proc+da);
			else                DOOR8_status = DOOR_DOWN;
		}


SetAnimation (anim_DOOR81, DOOR8_proc);		
SetAnimation (anim_DOOR82, DOOR8_proc);

SetAnimation (anim_DOOR83, DOOR8_proc);
}


Crew.WarnUserUMMUNotInstalled("ARROWHANGAR2");
if(GetAltitude()<11)
	{
		UpdateOnly2TimePerSecond+=simdt;
		if(UpdateOnly2TimePerSecond<1)
			return;
		UpdateOnly2TimePerSecond=0;
		double longitude=0;
		double latitude=0;
		double radius=0;
		GetEquPos (longitude,latitude,radius); // Get our position
		VECTOR3 th={longitude,latitude,240};
		if(GetAltitude()>5)
			th.z=0;
		SetRotDrag (th);
	}

}

	    
  










void ARROWHANGAR2::DefineAnimations(void)

{


  	static UINT EGrp20[2] = {GRP_door1,GRP_door1a};//DOOR1
   	static UINT EGrp21[2] = {GRP_door2,GRP_door2a};//DOOR2
   	static UINT EGrp22[2] = {GRP_door3,GRP_door3a};////DOOR3
   	static UINT EGrp23[2] = {GRP_door4,GRP_door4a};////DOOR4
   	static UINT EGrp24[2] = {GRP_door5,GRP_door5a};////DOOR5
   	static UINT EGrp25[2] = {GRP_door6,GRP_door6a};////DOOR6
   	static UINT EGrp26[2] = {GRP_door7,GRP_door7a};////DOOR7
	static UINT EGrp27[2] = {GRP_door8,GRP_door8a};//DOOR8
   	static UINT EGrp28[2] = {GRP_door9,GRP_door9a};//DOOR9
   	static UINT EGrp29[2] = {GRP_door10,GRP_door10a};//DOOR10
 //  	static UINT EGrp30[2] = {17,14};//frtgear2

   static MGROUP_TRANSLATE DOOR1(0,EGrp20, 2, _V(21.1,0,0));
   static MGROUP_TRANSLATE DOOR2(0,EGrp21, 2, _V(42.9,0,0));
   static MGROUP_TRANSLATE DOOR3(0,EGrp22, 2, _V(65.2,0,0));
   static MGROUP_TRANSLATE DOOR4(0,EGrp23, 2, _V(87,0,0));
   static MGROUP_TRANSLATE DOOR5(0,EGrp24, 2, _V(108.8,0,0));


   static MGROUP_TRANSLATE DOOR6(0,EGrp25, 2, _V(-108.8,0,0));

   static MGROUP_TRANSLATE DOOR7(0,EGrp26, 2, _V(-87,0,0));

 static MGROUP_TRANSLATE DOOR8(0,EGrp27, 2, _V(-65.2,0,0));

   static MGROUP_TRANSLATE DOOR9(0,EGrp28, 2, _V(-42.9,0,0));

      static MGROUP_TRANSLATE DOOR10(0,EGrp29, 2, _V(-21.1,0,0));

      
anim_DOOR1 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR1, .8, 1, &DOOR1) ;   
anim_DOOR2 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR2, .6, 1, &DOOR2) ;  
anim_DOOR3 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR3, .4, 1, &DOOR3) ;   
anim_DOOR4 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR4, .2, 1, &DOOR4) ;  
anim_DOOR5 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR5, 0, 1, &DOOR5) ;   

anim_DOOR6 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR6, 0, 1, &DOOR6) ;   
anim_DOOR7 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR7, .2, 1, &DOOR7) ;  
anim_DOOR8 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR8, .4, 1, &DOOR8) ;   
anim_DOOR9 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR9, .6, 1, &DOOR9) ;  
anim_DOOR10 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR10, .8, 1, &DOOR10) ;  


static UINT EGrp10[2] = {GRP_DOOR11,GRP_DOOR11A};//DOOR11
   	static UINT EGrp11[2] = {GRP_DOOR12,GRP_DOOR12A};//DOOR12
   	static UINT EGrp12[2] = {GRP_DOOR13,GRP_DOOR13A};////DOOR13
   	static UINT EGrp13[2] = {GRP_DOOR14,GRP_DOOR14A};////DOOR14
   	static UINT EGrp14[2] = {GRP_DOOR15,GRP_DOOR15A};////DOOR15
   	static UINT EGrp15[2] = {GRP_DOOR16,GRP_DOOR16A};////DOOR16
   	static UINT EGrp16[2] = {GRP_DOOR17,GRP_DOOR17A};////DOOR17
	static UINT EGrp17[2] = {GRP_DOOR18,GRP_DOOR18A};//DOOR18
   	static UINT EGrp18[2] = {GRP_DOOR19,GRP_DOOR19A};//DOOR19
   	static UINT EGrp19[2] = {GRP_DOOR20,GRP_DOOR20A};//DOOR20

   static MGROUP_TRANSLATE DOOR11(0,EGrp10, 2, _V(21.1,0,0));
   static MGROUP_TRANSLATE DOOR12(0,EGrp11, 2, _V(42.9,0,0));
   static MGROUP_TRANSLATE DOOR13(0,EGrp12, 2, _V(65.2,0,0));
   static MGROUP_TRANSLATE DOOR14(0,EGrp13, 2, _V(87,0,0));
   static MGROUP_TRANSLATE DOOR15(0,EGrp14, 2, _V(108.8,0,0));


   static MGROUP_TRANSLATE DOOR16(0,EGrp15, 2, _V(-108.8,0,0));

   static MGROUP_TRANSLATE DOOR17(0,EGrp16, 2, _V(-87,0,0));

 static MGROUP_TRANSLATE DOOR18(0,EGrp17, 2, _V(-65.2,0,0));

   static MGROUP_TRANSLATE DOOR19(0,EGrp18, 2, _V(-42.9,0,0));

      static MGROUP_TRANSLATE DOOR20(0,EGrp19, 2, _V(-21.1,0,0));

      
anim_DOOR11 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR11, .8, 1, &DOOR11) ;   
anim_DOOR12 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR12, .6, 1, &DOOR12) ;  
anim_DOOR13 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR13, .4, 1, &DOOR13) ;   
anim_DOOR14 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR14, .2, 1, &DOOR14) ;  
anim_DOOR15 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR15, 0, 1, &DOOR15) ;   

anim_DOOR16 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR16, 0, 1, &DOOR16) ;   
anim_DOOR17 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR17, .2, 1, &DOOR17) ;  
anim_DOOR18 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR18, .4, 1, &DOOR18) ;   
anim_DOOR19 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR19, .6, 1, &DOOR19) ;  
anim_DOOR20 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR20, .8, 1, &DOOR20) ;  

//reardoor
  	static UINT EGrp30[2] = {GRP_REARDOOR1B,GRP_reardoor2a};//DOOR1
   	static UINT EGrp31[2] = {GRP_REARDOOR2B,GRP_reardoor1a};//DOOR2
   	

   static MGROUP_TRANSLATE DOOR21(0,EGrp30, 2, _V(14,0,0));
   static MGROUP_TRANSLATE DOOR22(0,EGrp31, 2, _V(-14,0,0));

   anim_DOOR21 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR21, 0, 1, &DOOR21) ;   
anim_DOOR22 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR22, 0, 1, &DOOR22) ;  

//airlock inner

  	static UINT EGrp40[1] = {GRP_INNERAIRLOCK};//DOOR1
   	

   static MGROUP_TRANSLATE DOOR41(0,EGrp40, 1, _V(-2.78,0,0));

   anim_airlock1 = CreateAnimation (0); 
AddAnimationComponent  (anim_airlock1, 0, 1, &DOOR41) ;   

//extairlock   

  	static UINT EGrp50[2] = {GRP_EXTAIRLOCK1,GRP_EXTAIRLOCK2};//DOOR1
   	

   static MGROUP_TRANSLATE DOOR50(0,EGrp50, 2, _V(-2.78,0,0));

   anim_airlock2 = CreateAnimation (0); 
AddAnimationComponent  (anim_airlock2, 0, 1, &DOOR50) ;   

//cargo1

  	static UINT EGrp60[2] = {GRP_STORAGE11B,GRP_STORAGE11A};//DOOR1
   	static UINT EGrp61[2] = {GRP_STORAGE12B,GRP_STORAGE12A};//DOOR2
   	static UINT EGrp62[2] = {GRP_STORAGE13B,GRP_STORAGE13A};////DOOR3

   static MGROUP_TRANSLATE DOOR51(0,EGrp60, 2, _V(0,0,29.2));
   static MGROUP_TRANSLATE DOOR52(0,EGrp61, 2, _V(0,0,57.6));
   static MGROUP_TRANSLATE DOOR53(0,EGrp62, 2, _V(0,0,85.6));

      
anim_DOOR51 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR51, .6, 1, &DOOR51) ;   
anim_DOOR52 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR52, .3, 1, &DOOR52) ;  
anim_DOOR53 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR53, 0, 1, &DOOR53) ;   


//cargo2


  	static UINT EGrp70[2] = {GRP_STORAGE21B,GRP_STORAGE21A};//DOOR1
   	static UINT EGrp71[2] = {GRP_STORAGE22B,GRP_STORAGE22A};//DOOR2
   	static UINT EGrp72[2] = {GRP_STORAGE23B,GRP_STORAGE23A};////DOOR3
   	static UINT EGrp73[2] = {GRP_STORAGE24B,GRP_STORAGE24A};////DOOR3

   static MGROUP_TRANSLATE DOOR71(0,EGrp70, 2, _V(0,0,32.4));
   static MGROUP_TRANSLATE DOOR72(0,EGrp71, 2, _V(0,0,63.5));
   static MGROUP_TRANSLATE DOOR73(0,EGrp72, 2, _V(0,0,95.5));
   static MGROUP_TRANSLATE DOOR74(0,EGrp73, 2, _V(0,0,124.5));

      
anim_DOOR61 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR61, .75, 1, &DOOR71) ;   
anim_DOOR62 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR62, .5, 1, &DOOR72) ;  
anim_DOOR63 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR63, .25, 1, &DOOR73) ;   
anim_DOOR64 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR63, 0, 1, &DOOR74) ;   


//cargo3


  	static UINT EGrp80[2] = {GRP_STORAGE31B,GRP_STORAGE31A};//DOOR1
   	static UINT EGrp81[2] = {GRP_STORAGE32B,GRP_STORAGE32A};//DOOR2
   	static UINT EGrp82[2] = {GRP_STORAGE33B,GRP_STORAGE33A};////DOOR3
   	static UINT EGrp83[2] = {GRP_STORAGE34B,GRP_STORAG34C};////DOOR3

   static MGROUP_TRANSLATE DOOR81(0,EGrp80, 2, _V(0,0,32.4));
   static MGROUP_TRANSLATE DOOR82(0,EGrp81, 2, _V(0,0,63.5));
   static MGROUP_TRANSLATE DOOR83(0,EGrp82, 2, _V(0,0,95.5));
   static MGROUP_TRANSLATE DOOR84(0,EGrp83, 2, _V(0,0,124.5));

      
anim_DOOR71 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR71, .75, 1, &DOOR81) ;   
anim_DOOR72 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR72, .5, 1, &DOOR82) ;  
anim_DOOR73 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR73, .25, 1, &DOOR83) ;   
anim_DOOR74 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR74, 0, 1, &DOOR84) ;   




















//cargo4

  	static UINT EGrp90[2] = {GRP_STORAGE41A,GRP_STORAGE43B};//DOOR1
   	static UINT EGrp91[2] = {GRP_STORAGE42B,GRP_STORAGE42A};//DOOR2
   	static UINT EGrp92[2] = {GRP_STORAGE43A,GRP_STORAGE41B};////DOOR3

   static MGROUP_TRANSLATE DOOR91(0,EGrp90, 2, _V(0,0,29.2));
   static MGROUP_TRANSLATE DOOR92(0,EGrp91, 2, _V(0,0,57.6));
   static MGROUP_TRANSLATE DOOR93(0,EGrp92, 2, _V(0,0,85.6));


      
      
anim_DOOR81 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR81, .6, 1, &DOOR91) ;   
anim_DOOR82 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR82, .3, 1, &DOOR92) ;  
anim_DOOR83 = CreateAnimation (0); 
AddAnimationComponent  (anim_DOOR83, 0, 1, &DOOR93) ;   


//








}
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
return new ARROWHANGAR2 (hvessel, flightmodel);
}

DLLCLBK void ovcExit (VESSEL *vessel)
{
if (vessel) delete (ARROWHANGAR2*)vessel;
}



// --------------------------------------------------------------
// Keyboard interface handler (buffered key events)
// --------------------------------------------------------------
int ARROWHANGAR2::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)
{
	// only process keydown events
	if (!down) 
		return 0; 

//---------------------------------------------------------------------------
	// Ummu Key "E" perform the EVA of the selected member
	//
	// ADD REALISM: It's your responsabilities also to set ship's control accordingly to crew aboard.
	//				If you want to disable control if no one is aboard have a look at "SetADCtrlMode()"
	//				and "SetAttitudeMode()" functions of Orbiter. To disable thrusters set their fuel 
	//				ressource to NULL.
	if(key==OAPI_KEY_E&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		// PERFORM THE EVA, first we get is name with "GetCrewNameBySlotNumber" then we perform EVA with "EvaCrewMember"
		int Returned=Crew.EvaCrewMember(Crew.GetCrewNameBySlotNumber(SelectedUmmuMember));
		// we provide feedback to user (You can display a message on panel or wathewer)
		// here below all the return code possible:
		switch(Returned)
		{
		case TRANSFER_TO_DOCKED_SHIP_OK:
			sprintf(SendHudMessage(),"Transfer to docked ship Ok - %s transfered",
			Crew.GetLastEvaedCrewName());SelectedUmmuMember=0;
			break;
		case EVA_OK:
			sprintf(SendHudMessage(),"EVA OK - %s left the ship",
			Crew.GetLastEvaedCrewName());SelectedUmmuMember=0;
			break;
		case ERROR_NO_ONE_ON_BOARD:
			strcpy(SendHudMessage(),"Error, no one on board, unable to EVA");
			break;
		case ERROR_AIRLOCK_CLOSED:
			strcpy(SendHudMessage(),"Error, airlock is closed, unable to EVA");
			break;
		case ERROR_DOCKED_SHIP_HAVE_AIRLOCK_CLOSED:
			strcpy(SendHudMessage(),"Error, docked ship's airlock is closed, unable to transfer");
			break;
		case ERROR_DOCKED_SHIP_IS_FULL:
			strcpy(SendHudMessage(),"Error, docked ship is already full transfer failed");
			break;
		case ERROR_CREW_MEMBER_NOT_FOUND:
			strcpy(SendHudMessage(),"Error, no crew by this name in ship");
			break;
		case ERROR_DOCKEDSHIP_DONOT_USE_UMMU:
			strcpy(SendHudMessage(),"Error, docked ship do not use UMmu 2.0, ask author to add it");
			break;
		case ERROR_MISC_ERROR_EVAFAILED:
			strcpy(SendHudMessage(),"Misc error with UMMU install it again");
			break;
		}
		return TRUE;
	}

	//---------------------------------------------------------------------------
	// Ummu Key "1" Select next member This is just internal to the demo
	// you may do your own selection system by panel button, name etc etc
	if(key==OAPI_KEY_1&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		// we test there is someone aboard
		if(Crew.GetCrewTotalNumber()==0)
		{
			strcpy(SendHudMessage(),"Sorry no one aboard unable to select");	
			return 1;
		}

		// we test that we select existing member
		if(SelectedUmmuMember<Crew.GetCrewTotalNumber()-1)
			SelectedUmmuMember++;
		char * Name=Crew.GetCrewNameBySlotNumber(SelectedUmmuMember);
		sprintf(SendHudMessage(),"Slot %i  %s \"%s\" aged %i Selected for EVA or Transfer, please press \"E\" to EVA",
		SelectedUmmuMember,Crew.GetCrewMiscIdBySlotNumber(SelectedUmmuMember),
		Name,Crew.GetCrewAgeBySlotNumber(SelectedUmmuMember));
		return 1;
	}

	//---------------------------------------------------------------------------
	// Ummu Key "2" Select previous member This is just internal to the demo
	// you may do your own selection system by panel button
	if(key==OAPI_KEY_2&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		// we test there is someone aboard
		if(Crew.GetCrewTotalNumber()==0)
		{
			strcpy(SendHudMessage(),"Sorry no one aboard unable to select");	
			return 1;
		}
		if(SelectedUmmuMember>0)
			SelectedUmmuMember--;
		char * Name=Crew.GetCrewNameBySlotNumber(SelectedUmmuMember);
		sprintf(SendHudMessage(),"Slot %i %s \"%s\" aged %i Selected for EVA or Transfer"
			", please press \"E\" to EVA",SelectedUmmuMember,
			Crew.GetCrewMiscIdBySlotNumber(SelectedUmmuMember),Name,
			Crew.GetCrewAgeBySlotNumber(SelectedUmmuMember));
		return 1;
	}
	//---------------------------------------------------------------------------
	// Ummu Key "A" Switch the virtual UMMU airlock door on/off
	

	//---------------------------------------------------------------------------
	// Get some infos Name of ship and total soul aboard
	if(key==OAPI_KEY_S)
	{
		sprintf(SendHudMessage(),"%i souls aboard ship %s, %i seats available",
			Crew.GetCrewTotalNumber(),GetName(),2-Crew.GetCrewTotalNumber());
		return 1;
	}

	//---------------------------------------------------------------------------
	// ADD some Fun, Eject the guy, No check of all return code here to keep listing small and clear.
	// Notice eject function doesn't check airlock state at all.
	// GOOD IDEA: Get the Object's handle after ejection with function "GetObjHandleOfLastEVACrew"
	// and add to it one very small tank, thruster and smoke, then fire thruster (see pilot ejection of DGIV)
	// BAD IDEA: Not testing the handle returned by "GetObjHandleOfLastEVACrew" before using may
	// cause a CTD if by any bad luck the pointer is invalid (handle==NULL)
	if(key==OAPI_KEY_ESCAPE&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		if(Crew.EjectCrewMember(Crew.GetCrewNameBySlotNumber(SelectedUmmuMember))==EVA_OK)
			sprintf(SendHudMessage(),"%s EJECTED",Crew.GetLastEvaedCrewName());
		SelectedUmmuMember=0;
		return 1;
	}

	//---------------------------------------------------------------------------
	// Use a different Mesh (type "C" then EVA someone)
	// better idea is to use the new UMMU Id definition
	// look readme.txt in folder "config/UMMUIdConfig"
	if(key==OAPI_KEY_C)
	{
		Crew.SetAlternateMeshToUseForEVASpacesuit("mmu");	// the stock mmu of orbiter located in "meshes/mmu.msh"
		strcpy(SendHudMessage(),"Mesh changed");
		return 1;
	}

	// THIS IS FOR ADDING CREW SEE PDF doc "Allow user to add crew to your ship 
	// without scenery editor"
	if(key==OAPI_KEY_M&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		AddUMmuToVessel(TRUE);
	}
	//---------------------------------------------------------------------------
	


if(key==OAPI_KEY_G)
{
RevertDOOR();
}
if(key==OAPI_KEY_O)
{
RevertDOOR1();
}
if(key==OAPI_KEY_K)
{
RevertDOOR2();
}

if(key==OAPI_KEY_3)
{
RevertDOOR5();
}
if(key==OAPI_KEY_4)
{
RevertDOOR6();
}
if(key==OAPI_KEY_5)
{
RevertDOOR7();
}
if(key==OAPI_KEY_6)
{
RevertDOOR8();
}

	
return 0;
}

// ====================================================================
// clbkVisualCreated used to display UMMU initialisation message 
// because oapiDebugString() doesn't work in clbkSetClassCap
// ====================================================================
void ARROWHANGAR2::clbkVisualCreated (VISHANDLE vis, int refcount)
{   
	MainExternalMeshVisual = GetMesh(vis,0);
	
}
// ==============================================================
// Visual destroyed
// ==============================================================
void ARROWHANGAR2::clbkVisualDestroyed (VISHANDLE vis, int refcount)
{
	MainExternalMeshVisual = 0;	
}



// ========================================================================
// clbkLoadState function of Orbiter - Save scenario when user exit Orbiter 
// ========================================================================

//------------------------------------------------------------------------------------------------------------------
// This below is not mandatory for UMmu to run.
//------------------------------------------------------------------------------------------------------------------

// ==========================================================
// USING TEXT INTERFACE FOR CREW MANAGMENT (not a good idea)
//
// This is the custom functions that send oapiDebugString() Message on screen
// it simply avoid that the message is sent if our ship don't have the focus
// and ensure that message will be reminded when we focus back on ship
// the Orbiter function below clbkFocusChanged is used to erase message when
// we lost focus.
// It's a bad idea to do an UMmu user interface only by using Debug screen message 
// but if you can't do better you may use this system.
///////////////////////////////////////////////////////////////////////////////////////////

// --------------------------------------------------------------
// Orbiter's HUD callback
// used to display UMMU's message see PDF doc:
// "Example of feedback method by HUD Display"
// --------------------------------------------------------------
void ARROWHANGAR2::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, HDC hDC)
{
	// draw the default HUD
	VESSEL2::clbkDrawHUD (mode, hps, hDC);

	// UMmu display messages
	if(dHudMessageDelay>0)
	{
		TextOut (hDC,5,hps->H/60*15,cUmmuHudDisplay,strlen(cUmmuHudDisplay));
		dHudMessageDelay-=oapiGetSimStep();
		if(dHudMessageDelay<0)
			dHudMessageDelay=0;
	}
}
void ARROWHANGAR2::clbkSaveState(FILEHANDLE scn)
{
char cbuf[256];
	



	// ORBITER, default vessel parameters
	SaveDefaultState (scn);

		sprintf (cbuf, "%d %0.4f", DOOR0_status, DOOR0_proc);
		oapiWriteScenario_string (scn, "DOOR0", cbuf);

		sprintf (cbuf, "%d %0.4f", DOOR1_status, DOOR1_proc);
		oapiWriteScenario_string (scn, "DOOR1", cbuf);

		sprintf (cbuf, "%d %0.4f", DOOR2_status, DOOR2_proc);
		oapiWriteScenario_string (scn, "DOOR2", cbuf);

		sprintf (cbuf, "%d %0.4f", DOOR3_status, DOOR3_proc);
		oapiWriteScenario_string (scn, "DOOR3", cbuf);

		sprintf (cbuf, "%d %0.4f", DOOR4_status, DOOR4_proc);
		oapiWriteScenario_string (scn, "DOOR4", cbuf);

		sprintf (cbuf, "%d %0.4f", DOOR5_status, DOOR5_proc);
		oapiWriteScenario_string (scn, "DOOR5", cbuf);

		sprintf (cbuf, "%d %0.4f", DOOR6_status, DOOR6_proc);
		oapiWriteScenario_string (scn, "DOOR6", cbuf);

		sprintf (cbuf, "%d %0.4f", DOOR7_status, DOOR7_proc);
		oapiWriteScenario_string (scn, "DOOR7", cbuf);

		sprintf (cbuf, "%d %0.4f", DOOR8_status, DOOR8_proc);
		oapiWriteScenario_string (scn, "DOOR8", cbuf);



	// UMmu, this will save all our members of crew in scenario
	// as airlock state depend of your ship (you may have a real animated airlock)
	// it's your responsabilites to save and reload UMmu's airlock state.
	Crew.SaveAllMembersInOrbiterScenarios(scn);
}
void ARROWHANGAR2::clbkLoadStateEx (FILEHANDLE scn, void *status)
{
	char *line;
	while (oapiReadScenario_nextline (scn, line)) 
	{
		

		// UMMU, Load all saved member from scenario. 
		// Return TRUE when a "UMMUCREW" line is encountered 
		// FALSE when it must pass this line to you or default parser
		if(Crew.LoadAllMembersFromOrbiterScenario(line)==TRUE)
			continue;


	      if (!_strnicmp (line, "DOOR0", 5)) {
   sscanf (line+4, "%d%lf", &DOOR0_status, &DOOR0_proc);
			}

			
		  	      if (!_strnicmp (line, "DOOR1", 5)) {
   sscanf (line+5, "%d%lf", &DOOR1_status, &DOOR1_proc);
			}


      if (!_strnicmp (line, "DOOR2", 5)) {
   sscanf (line+5, "%d%lf", &DOOR2_status, &DOOR2_proc);
			}
	        if (!_strnicmp (line, "DOOR3", 5)) {
   sscanf (line+5, "%d%lf", &DOOR3_status, &DOOR3_proc);
			}
			      if (!_strnicmp (line, "DOOR4", 5)) {
   sscanf (line+5, "%d%lf", &DOOR4_status, &DOOR4_proc);
			}
				        if (!_strnicmp (line, "DOOR5", 5)) {
   sscanf (line+5, "%d%lf", &DOOR5_status, &DOOR5_proc);
			}
						      if (!_strnicmp (line, "DOOR6", 5)) {
   sscanf (line+5, "%d%lf", &DOOR6_status, &DOOR6_proc);
			}

							       if (!_strnicmp (line, "DOOR7", 5)) {
   sscanf (line+5, "%d%lf", &DOOR7_status, &DOOR7_proc);
			}
						      if (!_strnicmp (line, "DOOR8", 5)) {
   sscanf (line+5, "%d%lf", &DOOR8_status, &DOOR8_proc);
			}





		        
		// ORBITER, unrecognised option - pass to Orbiter's generic parser
		ParseScenarioLineEx (line, status);
	}
	


SetAnimation (anim_DOOR1, DOOR0_proc);		
SetAnimation (anim_DOOR2, DOOR0_proc);

SetAnimation (anim_DOOR3, DOOR0_proc);
SetAnimation (anim_DOOR4, DOOR0_proc);
SetAnimation (anim_DOOR5, DOOR0_proc);
SetAnimation (anim_DOOR6, DOOR0_proc);
SetAnimation (anim_DOOR7, DOOR0_proc);
SetAnimation (anim_DOOR8, DOOR0_proc);
SetAnimation (anim_DOOR9, DOOR0_proc);
SetAnimation (anim_DOOR10, DOOR0_proc);


SetAnimation (anim_DOOR11, DOOR1_proc);		
SetAnimation (anim_DOOR12, DOOR1_proc);
SetAnimation (anim_DOOR13, DOOR1_proc);
SetAnimation (anim_DOOR14, DOOR1_proc);
SetAnimation (anim_DOOR15, DOOR1_proc);
SetAnimation (anim_DOOR16, DOOR1_proc);
SetAnimation (anim_DOOR17, DOOR1_proc);
SetAnimation (anim_DOOR18, DOOR1_proc);
SetAnimation (anim_DOOR19, DOOR1_proc);
SetAnimation (anim_DOOR20, DOOR1_proc);

SetAnimation (anim_DOOR21, DOOR2_proc);		
SetAnimation (anim_DOOR22, DOOR2_proc);

SetAnimation (anim_airlock1, DOOR3_proc);		
SetAnimation (anim_airlock2, DOOR4_proc);

SetAnimation (anim_DOOR51, DOOR5_proc);		
SetAnimation (anim_DOOR52, DOOR5_proc);
SetAnimation (anim_DOOR53, DOOR5_proc);

SetAnimation (anim_DOOR61, DOOR6_proc);		
SetAnimation (anim_DOOR62, DOOR6_proc);
SetAnimation (anim_DOOR63, DOOR6_proc);
SetAnimation (anim_DOOR64, DOOR6_proc);

SetAnimation (anim_DOOR71, DOOR7_proc);		
SetAnimation (anim_DOOR72, DOOR7_proc);
SetAnimation (anim_DOOR73, DOOR7_proc);
SetAnimation (anim_DOOR74, DOOR7_proc);


SetAnimation (anim_DOOR81, DOOR8_proc);		
SetAnimation (anim_DOOR82, DOOR8_proc);
SetAnimation (anim_DOOR83, DOOR8_proc);



	// as airlock state depend of your ship (you may have a real animated airlock)
	// it's your responsabilites to save and reload UMmu airlock state and set it accordingly
	// to your airlock. IE: call here "SetAirlockDoorState" TRUE of FALSE depend
	// of your animated airlock
}
bool UMmuCrewAddCallback(void *id, char *str, void *data)
{
	if(strlen(str)<2||strlen(str)>38)
		return false;
	char *cPtr=(char*)data;	if(*cPtr==2){*cPtr=3;strcpy(cPtr+2,str);}
	else if(*cPtr==4){*cPtr=5;strcpy(cPtr+42,str);}
	else if(*cPtr==6){*cPtr=7;strcpy(cPtr+82,str);}return true;
}
void ARROWHANGAR2::AddUMmuToVessel(BOOL bStartAdding)
{
	if(bStartAdding==FALSE&&cAddUMmuToVessel[0]==0)
		return;
	if(bStartAdding==TRUE){
		int salut=sizeof(cAddUMmuToVessel);
		memset(cAddUMmuToVessel,0,sizeof(cAddUMmuToVessel));
		cAddUMmuToVessel[0]=1;
	}
	else if(cAddUMmuToVessel[0]==1){
		cAddUMmuToVessel[0]=2;
		oapiOpenInputBox ("Enter new crew's name (or escape)",UMmuCrewAddCallback,0,30,(void*)cAddUMmuToVessel);
	}
	else if(cAddUMmuToVessel[0]==3){
		cAddUMmuToVessel[0]=4;
		oapiOpenInputBox ("Enter crew's age",UMmuCrewAddCallback,0,30,(void*)cAddUMmuToVessel);
	}
	else if(cAddUMmuToVessel[0]==5){
		cAddUMmuToVessel[0]=6;
		oapiOpenInputBox ("Enter function (Capt,Sec,Vip,Sci,Doc,Tech,Crew,Pax)",UMmuCrewAddCallback,0,30,(void*)cAddUMmuToVessel);
	}
	else if(cAddUMmuToVessel[0]==7){
		cAddUMmuToVessel[0]=0;
		int Age=max(5,min(100,atoi(&cAddUMmuToVessel[42])));
		if(Crew.AddCrewMember(&cAddUMmuToVessel[2],Age,70,70,&cAddUMmuToVessel[82])==TRUE){
			sprintf(SendHudMessage(),"Crew \"%s\" aged %i added to vessel",&cAddUMmuToVessel[2],Age);
		}
		else{
			strcpy(SendHudMessage(),"ERROR: Crew not added (vessel full?)");
		}
	}
}


 void ARROWHANGAR2::RevertDOOR (void)
{
	DOOR0_status = ((DOOR0_status == DOOR_UP || DOOR0_status == DOOR_RAISING) ?
		DOOR_LOWERING : DOOR_RAISING);
}

 void ARROWHANGAR2::RevertDOOR1 (void)
{
	DOOR1_status = ((DOOR1_status == DOOR_UP || DOOR1_status == DOOR_RAISING) ?
		DOOR_LOWERING : DOOR_RAISING);
}
void ARROWHANGAR2::RevertDOOR2 (void)
{
	DOOR2_status = ((DOOR2_status == DOOR_UP || DOOR2_status == DOOR_RAISING) ?
		DOOR_LOWERING : DOOR_RAISING);
}
void ARROWHANGAR2::RevertDOOR3 (void)
{
	DOOR3_status = ((DOOR3_status == DOOR_UP || DOOR3_status == DOOR_RAISING) ?
		DOOR_LOWERING : DOOR_RAISING);
}
void ARROWHANGAR2::RevertDOOR4 (void)
{
	DOOR4_status = ((DOOR4_status == DOOR_UP || DOOR4_status == DOOR_RAISING) ?
		DOOR_LOWERING : DOOR_RAISING);
}
void ARROWHANGAR2::RevertDOOR5 (void)
{
	DOOR5_status = ((DOOR5_status == DOOR_UP || DOOR5_status == DOOR_RAISING) ?
		DOOR_LOWERING : DOOR_RAISING);
}
void ARROWHANGAR2::RevertDOOR6 (void)
{
	DOOR6_status = ((DOOR6_status == DOOR_UP || DOOR6_status == DOOR_RAISING) ?
		DOOR_LOWERING : DOOR_RAISING);
}
void ARROWHANGAR2::RevertDOOR7 (void)
{
	DOOR7_status = ((DOOR7_status == DOOR_UP || DOOR7_status == DOOR_RAISING) ?
		DOOR_LOWERING : DOOR_RAISING);
}
void ARROWHANGAR2::RevertDOOR8 (void)
{
	DOOR8_status = ((DOOR8_status == DOOR_UP || DOOR8_status == DOOR_RAISING) ?
		DOOR_LOWERING : DOOR_RAISING);
}